#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/* Blackjack is all about getting as close as you can to this
 * TARGET_SCORE, without exceeding it. Use this constant in your code
 * instead of 21.
 */
const int TARGET_SCORE = 21;

/* A particular card is represented by two integers: the rank and the
 * suit. A rank is a number from 1 to 13. The numbers 2 through 10
 * represent the numeric cards, and the other four cards use the
 * numbers assigned to them below.
 */
const int ACE = 1;
const int JACK = 11;
const int QUEEN = 12;
const int KING = 13;

/* The suit is also represented by an integer, in the range zero to
 * three. The assignment of numbers to suits is arbitrary (the order
 * doesn't matter).
 */
const int CLUBS = 0;
const int HEARTS = 1;
const int SPADES = 2;
const int DIAMONDS = 3;

/* TODO: This function takes two integers representing a single
 * playing card, and it should print out a human-readable
 * representation of the card, such as "Ace of diamonds". Hint: you
 * probably want to use a couple of 'switch' statements.
 */
void print_card(int rank, int suit)
{
}

/* This is a test program for print_card. When your print_card
 * function is working, you should see the outputs given in the
 * comments. Do not modify this function.
 */
void test_print_card()
{
    printf("######################## test_print_card\n");
    print_card(2, DIAMONDS);     // 2 of diamonds
    print_card(10, CLUBS);       // 10 of clubs
    print_card(KING, HEARTS);    // King of hearts
    print_card(ACE, SPADES);     // Ace of spades
    print_card(QUEEN, DIAMONDS); // Queen of diamonds
    print_card(5, CLUBS);        // 5 of clubs
}

/* TODO: This function randomly selects a card by rolling a random
 * rank and suit. Remember, a rank is an integer from 1 to 13, and
 * suit ranges from 0 to 3. It should also print the card to the
 * screen using print_card, and then return the rank. (The suit is not
 * used for scoring in Blackjack, so we don't need to return that.)
 */
int deal_one_card()
{
    return JACK; // temporary
}

/* This is a test program for deal_one_card. It just runs it 5 times,
 * and you should see 5 different cards printed out. (Actually it's
 * possible to get the same card twice, if we're moderately unlucky.)
 * Underneath each card, it will print the rank returned by
 * deal_one_card, and that should match the rank you printed (KING ==
 * 13, etc.) Do not modify this function.
 */
void test_deal_one_card()
{
    printf("######################## test_deal_one_card\n");
    for(int i = 0; i < 5; i++)
    {
        int rank = deal_one_card();
        printf("  (returned rank %d)\n", rank);
    }
}

/* TODO: This function converts a rank into a score for Blackjack. In
 * the game, numeric cards have their value, for example a 3 is worth
 * 3 points and a 9 is worth 9 points. The "face cards": Jack, Queen,
 * King, are each worth 10 points. Finally, we'll count the Ace as 11
 * points. In real Blackjack, aces can be counted high or low (11
 * points vs 1 point), but we'll ignore that rule for now. Return the
 * score appropriate for the given rank.
 */
int rank_to_score(int rank)
{
    return 0; // temporary
}

/* Test program for rank_to_score. This program uses an array, which
 * is a feature we haven't covered yet. But it will report if your
 * rank_to_score function is correct or erroneous for each possible
 * rank. Do not modify this function.
 */
void test_rank_to_score()
{
    printf("######################## test_rank_to_score\n");
    int correct_scores[] =
        {
            -1, 11,                     // invalid, ace
            2, 3, 4, 5, 6, 7, 8, 9, 10, // number cards
            10, 10, 10                  // face cards
        };
    for(int i = ACE; i <= KING; i++) 
    {
        int s = rank_to_score(i);
        printf("rank %d -> score %d (%s)\n", i, s,
               s == correct_scores[i]? "correct" : "error");
    }
}

/* TODO: This function will ask the user whether they want to hit
 * (take one more card) or stay (keep what they have). It should
 * validate the user's input and if they enter an invalid choice, it
 * should keep asking until they get it right. It returns either 1
 * (to hit) or 0 (to stay).
 */
int ask_user_hit()
{
    return 0; // temporary
}

/* TODO: This function should print a message related to the final
 * score of the game (which is a parameter). If that score is exactly
 * the TARGET_SCORE, then announce that the player has won
 * ("BLACKJACK!"). If it is bigger than the target, announce that they
 * lost ("BUST!"). Either way, you should kindly thank the user for
 * playing our game.
 */
void maybe_announce_result(int score)
{
}

/* This is a test program for maybe_announce_result. The desired
 * output of each call is given in the comments. Do not modify this
 * function.
 */
void test_maybe_announce_result()
{
    printf("######################## test_maybe_announce_result\n");
    maybe_announce_result(TARGET_SCORE); // BLACKJACK! Thanks.
    maybe_announce_result(22);           // BUST! Thanks.
    maybe_announce_result(25);           // BUST! Thanks.
    maybe_announce_result(10);           // Thanks.
}

/* Here is a function which implements the entire flow of the
 * Blackjack game, using all the functions above. It deals two cards,
 * and then adds their scores. Then it asks the user to hit or stay
 * until the game is over, and announces the final result. Do not
 * modify this function.
 */
void play_game()
{
    printf("######################## starting game.\n");
    int rank, score;
    // First two cards.
    rank = deal_one_card();
    score = rank_to_score(rank);
    rank = deal_one_card();
    score += rank_to_score(rank);
    printf("Your score: %d\n", score);
    // Loop as long as user wants (or until we hit 21).
    while(score < TARGET_SCORE && ask_user_hit())
    {
        rank = deal_one_card();
        score += rank_to_score(rank);
        printf("Your score: %d\n", score);
    }
    // All done. Was it Blackjack or Bust?
    maybe_announce_result(score);
}

/* Run all of the test programs above, in sequence. If you want to
 * skip certain tests to concentrate on others, you can just comment
 * them out within this function.
 */
void all_tests()
{
    //test_print_card();
    //test_deal_one_card();
    test_rank_to_score();
    //test_maybe_announce_result();
}

/* The main program. You can comment out play_game if you just want to
 * run the tests, or comment out all_tests if you just want to play
 * the game.
 */
int main()
{
    // Initialize RNG.
    srand(time(NULL));
    all_tests();
//    play_game();
}
